
import gpioc._gpioc as _gpio

class Pin:
    IN = 0
    OUT = 1
    LOW = 0
    HIGH = 1
    PULL_NONE = 0
    PULL_UP = 1
    PULL_DOWN = 2

    id = None
    _value = LOW
    _mode = IN

    def _setup(self, gpio_num, dir, pull_up_down=None ):
        if dir == _gpio.f_INPUT :
            _gpio.set_mode(gpio_num, _gpio.f_INPUT)
            if pull_up_down == self.PULL_UP:
                _gpio.set_pullUpDn(gpio_num, _gpio.f_pullUp)
            elif pull_up_down == self.PULL_DOWN:
                _gpio.set_pullUpDn(gpio_num, _gpio.f_pullDown)
            else:
                _gpio.set_pullUpDn(gpio_num, _gpio.f_pull_OFF)
        if dir == _gpio.f_OUTPUT :
            _gpio.set_mode(gpio_num, _gpio.f_OUTPUT)
                
    # self.output(self.id, val)
    def _output(self, gpio_num, val):
        _gpio.write(gpio_num, val)

    # return _gpio.input(self.id)
    def _input(self, gpio_num):
        return _gpio.read(gpio_num)

    def __init__(self, pin_name):
        if isinstance(pin_name, tuple):
            self.id = int(pin_name[1])
        else:
            self.id = int(pin_name)

    def __repr__(self):
        return str(self.id)

    def __eq__(self, other):
        return self.id == other

    def init(self, mode=IN, pull=None):
        """Initialize the Pin"""
        if mode is not None:
            if mode == self.IN:
                self._mode = self.IN
                # GPIO.setup(self.id, GPIO.IN)
                self._setup(self.id, self.IN)

            elif mode == self.OUT:
                self._mode = self.OUT
                # GPIO.setup(self.id, GPIO.OUT)
                self._setup(self.id, self.OUT)

            else:
                raise RuntimeError("Invalid mode for pin: %s" % self.id)
        if pull is not None:
            if self._mode != self.IN:
                raise RuntimeError("Cannot set pull resistor on output")
            if pull == self.PULL_UP:
                # GPIO.setup(self.id, GPIO.IN, pull_up_down=GPIO.PUD_UP)
                self._setup(self.id, self.IN, pull_up_down=self.PULL_UP)
                

            elif pull == self.PULL_DOWN:
                # GPIO.setup(self.id, GPIO.IN, pull_up_down=GPIO.PUD_DOWN)
                self._setup(self.id, self.IN, pull_up_down=self.PULL_DOWN)

                pass
            else:
                raise RuntimeError("Invalid pull for pin: %s" % self.id)
            
    def value(self, val=None):
            """Set or return the Pin Value"""
            if val is not None:
                if val == self.LOW:
                    self._value = val
                    # GPIO.output(self.id, val)
                    self._output(self.id, val)

                elif val == self.HIGH:
                    self._value = val
                    # GPIO.output(self.id, val)
                    self._output(self.id, val)

                else:
                    raise RuntimeError("Invalid value for pin")
                return None
            return self._input(self.id)


import gpioc._softpwm as _pwm
import gpioc as _gpio

class PWMOut_father:
    """Base Pulse Width Modulation Output Class"""
    _gpio_num = 0
    _chip = 0 
    def __init__(self, pin, *, frequency=500, duty_cycle=0, variable_frequency=False):
        self._gpio_num = pin.id

        pin.init(pin.OUT)
        _pwm.switch_chip(self._chip)
        _pwm.set_frequency(self._gpio_num, frequency)
        _pwm.set_duty_cycle(self._gpio_num, duty_cycle)
        _pwm.start(self._gpio_num)
        

    def __del__(self):
        self.deinit()

    def __enter__(self):
        return self

    def __exit__(self, t, value, traceback):
        self.deinit()

    def deinit(self):
        _pwm.stop(self._gpio_num)

    @property
    def duty_cycle(self):
        """The PWM's output duty cycle, 16-bit."""
        return _pwm.get_duty_cycle(self._gpio_num)
        
    @duty_cycle.setter
    def duty_cycle(self, duty_cycle):
        _pwm.set_duty_cycle(self._gpio_num, int(duty_cycle))

    @property
    def frequency(self):
        return _pwm.get_frequency(self._gpio_num)

    @frequency.setter
    def frequency(self, frequency):
        _pwm.set_frequency(self._gpio_num, int(frequency))




from gpioc._pin import Pin
from gpioc._pwm import PWMOut_father as _PWM

import gpioc._gpioc as _common

class Pwm(_PWM):
    _chip = _common.CHIP_H616

# class Pin(_PIN):

    # def _setup(self, gpio_num, dir, pull_up_down=None ):
    #     if dir == _gpio.f_INPUT :
    #         _gpio.set_mode(gpio_num, _gpio.f_INPUT)
    #         if pull_up_down == self.PULL_UP:
    #             _gpio.set_pullUpDn(gpio_num, _gpio.f_pullUp)
    #         elif pull_up_down == self.PULL_DOWN:
    #             _gpio.set_pullUpDn(gpio_num, _gpio.f_pullDown)
    #         else:
    #             _gpio.set_pullUpDn(gpio_num, _gpio.f_pull_OFF)
    #     if dir == _gpio.f_OUTPUT :
    #         _gpio.set_mode(gpio_num, _gpio.f_OUTPUT)
                
    # # self.output(self.id, val)
    # def _output(self, gpio_num, val):
    #     _gpio.write(gpio_num, val)

    # # return _gpio.input(self.id)
    # def _input(self, gpio_num):
    #     return _gpio.read(gpio_num)
    

PC0 = Pin(64)
PC1 = Pin(65)
PC2 = Pin(66)
PC3 = Pin(67)
PC4 = Pin(68)
PC5 = Pin(69)
PC6 = Pin(70)
PC7 = Pin(71)
PC8 = Pin(72)
PC9 = Pin(73)
PC10 = Pin(74)
PC11 = Pin(75)
PC12 = Pin(76)
PC13 = Pin(77)
PC14 = Pin(78)
PC15 = Pin(79)

PF0 = Pin(160)
PF1 = Pin(161)
PF2 = Pin(162)
PF3 = Pin(163)
PF4 = Pin(164)
PF5 = Pin(165)
PF6 = Pin(166)

PG0 = Pin(192)
PG1 = Pin(193)
PG2 = Pin(194)
PG3 = Pin(195)
PG4 = Pin(196)
PG5 = Pin(197)
PG6 = Pin(198)
PG7 = Pin(199)
PG8 = Pin(200)
PG9 = Pin(201)
PG10 = Pin(202)
PG11 = Pin(203)
PG12 = Pin(204)
PG13 = Pin(205)
PG14 = Pin(206)
PG15 = Pin(207)
PG16 = Pin(208)
PG17 = Pin(209)
PG18 = Pin(210)
PG19 = Pin(211)

PH0 = Pin(224)
PH1 = Pin(225)
PH2 = Pin(226)
PH3 = Pin(227)
PH4 = Pin(228)
PH5 = Pin(229)
PH6 = Pin(230)
PH7 = Pin(231)
PH8 = Pin(232)
PH9 = Pin(233)
PH10 = Pin(234)

PI0 = Pin(256)
PI1 = Pin(257)
PI2 = Pin(258)
PI3 = Pin(259)
PI4 = Pin(260)
PI5 = Pin(261)
PI6 = Pin(262)
PI7 = Pin(263)
PI8 = Pin(264)
PI9 = Pin(265)
PI10 = Pin(266)
PI11 = Pin(267)
PI12 = Pin(268)
PI13 = Pin(269)
PI14 = Pin(270)
PI15 = Pin(271)
PI16 = Pin(272)


from gpioc import *
import time
key = PC12
led = PC13
key.init(key.IN,key.PULL_UP)
led.init(led.OUT)

PC12.init(PC12.IN,PC12.PULL_UP)
PC13.init(PC13.OUT)

while True:
    time.sleep(0.1)
    if key.value() == 0:
        led.value(1)
    else:
        led.value(0)


from setuptools import setup, Extension, find_packages
classifiers = [
    'Development Status :: 3 - Alpha',
    'Intended Audience :: Developers',
    'Topic :: Software Development :: Build Tools',
    'License :: OSI Approved :: MIT License',
    'Programming Language :: Python :: 3',
]


setup(
    name="gpioc",
    version="0.2",
    author="sc-bin",
    author_email="3335447573@qq.com",
    description="A module to control GPIO",
    long_description = open('README.md').read(),
    license          = 'MIT',
    url="https://github.com/sc-bin/gpioc", 
    packages=find_packages(),

    ext_modules = [
                #    Extension('gpioc._h616', ['chips/h616.c', 'chips/h616_py.c']),
                   Extension('gpioc._softpwm', ['src/softpwm.c', 'src/softpwm_py.c',  'chips/h616.c']),
                   Extension('gpioc._gpioc', ['src/gpioc_py.c', 'src/gpioc.c', 'chips/h616.c']),
                   ],

)
